// =========================| Third Party Node Template |=========================

#pragma once
#include <JuceHeader.h>
#include "src/rubberband3/rubberband/RubberBandStretcher.h"
#include "src/rubberband3/single/RubberBandSingle.cpp"

namespace project
{
using namespace juce;
using namespace hise;
using namespace scriptnode;

// =================| The node class with all required callbacks |=================

template <int NV> struct rubber_band: public data::base
{
	// Metadata Definitions ------------------------------------------------------
	
	SNEX_NODE(rubber_band);
	
	struct MetadataClass
	{
		SN_NODE_ID("rubber_band");
	};
	
	// set to true if you want this node to have a modulation dragger
	static constexpr bool isModNode() { return false; };
	static constexpr bool isPolyphonic() { return NV > 1; };
	// set to true if your node produces a tail
	static constexpr bool hasTail() { return false; };
	// Undefine this method if you want a dynamic channel count
	static constexpr int getFixChannelAmount() { return 2; };
	
	// Define the amount and types of external data slots you want to use
	static constexpr int NumTables = 0;
	static constexpr int NumSliderPacks = 0;
	static constexpr int NumAudioFiles = 0;
	static constexpr int NumFilters = 0;
	static constexpr int NumDisplayBuffers = 0;
			
	std::unique_ptr<RubberBand::RubberBandStretcher> rb;
	
	// Scriptnode Callbacks ------------------------------------------------------
	
	void prepare(PrepareSpecs specs)
	{
		rb = std::make_unique<RubberBand::RubberBandStretcher> (specs.sampleRate,
		                        specs.numChannels,
														RubberBand::RubberBandStretcher::Option::OptionProcessRealTime
													);	
	}
	
	void reset()
	{

	}
	
	void handleHiseEvent(HiseEvent& e)
	{
		
	}
	
	template <typename T> void process(T& data)
	{
		static constexpr int NumChannels = getFixChannelAmount();
		// Cast the dynamic channel data to a fixed channel amount
		auto& fixData = data.template as<ProcessData<NumChannels>>();
		
		// Create a FrameProcessor object
		auto fd = fixData.toFrameData();
		
		auto ptrs = data.getRawDataPointers();
		int numSamples = data.getNumSamples();
		
		while (rb->available() < numSamples)
		{
			rb->process(ptrs, numSamples, false);
		}
	
		rb->retrieve(ptrs, numSamples);
		
	}
	
	template <typename T> void processFrame(T& data)
	{		

	}
	
	int handleModulation(double& value)
	{		
		return 0;		
	}
	
	void setExternalData(const ExternalData& data, int index)
	{
		
	}
	// Parameter Functions -------------------------------------------------------
	
	template <int P> void setParameter(double v)
	{
		if (P == 0)
		{
			rb->setTimeRatio(v);
		}
		
	}
	
	void createParameters(ParameterDataList& data)
	{
		{
			// Create a parameter like this
			parameter::data p("Time", { 0.25, 2.0 });
			registerCallback<0>(p);
			p.setDefaultValue(1.0);
			data.add(std::move(p));
		}
	}

};
}


